// Fraktaltyp Barnsley Var. C
// p[0] = pixel.x
// p[1] = pixel.y
#include <stdio.h>
#include <stdlib.h>
#include "barnsc.h"
#include "barnsc.moc"
#include "komplex.h"
#define TRS klocale->translate

double BarnsleyCPar[2] = {0.1, 0.36};

void BarnsleyC::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void BarnsleyC::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -3;     // Gre des Fraktals
   *x2 =  3;
   *y1 = -2;
   *y2 =  2;
}

double BarnsleyC::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return BarnsleyCPar[Index];
}

int BarnsleyC::CalcPoint(int x, int y)
{
double rx = li+dx*x;
double ry = un+dy*y;
int    iter=0;

struct komplex z,z2,h,c;
   
   z.re = rx;
   z.im = ry;
   if (l[0]) {
      c.re = rx;  // Mandeltype
      c.im = ry;
   } else {
      c.re = p[0];  // Juliatype
      c.im = p[1];
   }
   do {
      z2 = cmul(z,z);  // z^2
      if (z.re>=0.0) {
         h.re = z.re*z.re-z.im*z.im - 1;
         h.im = 2*z.re*z.im;
      } else {
         h.re = z.re*z.re-z.im*z.im-1 + c.re*z.re;
         h.im = 2*z.re*z.im + c.im*z.re;
      }
      z.re = h.re;
      z.im = h.im;
      iter++;
   } while ((iter<MaxIter) && ConvTest(z.re*z.re,z.im*z.im) );

   return PostCompute(iter,z.re,z.im);
}

char* BarnsleyC::GetFracInfo(void)
{
   return 
"Formula:\n\
z=Pixel;\n\
if (Juliafractal)  c=(p0,p1)  else  c=Pixel\n\
loop {\n\
   if (real(z)*imag(c) + real(c)*imag(z) >= 0)\n\
      z = (z-1)*c\n\
   else\n\
      z = (z+1)*c\n\
} until (z^2>Bail)";
}

int BarnsleyC::SetupFrac(BaseFract* Frac) // type specific-setup
{
BarnsleyCDlg MDlg = BarnsleyCDlg(Frac, 0, "Barnsley Type C settings");

   if (MDlg.exec()) {
      return MDlg.SetSpecificParams();
   } else
      return FALSE;
}


// -----------------------------------------

BarnsleyCDlg::BarnsleyCDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[32];
   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Perturbation of X(0)"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   ConstX = new QLineEdit(this, "OffsetX");
   ConstX->setGeometry(130,50,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[0]);
   ConstX->setText(Strg);
   ConstY = new QLineEdit(this, "OffsetY");
   ConstY->setGeometry(130,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[1]);
   ConstY->setText(Strg);

   QLabel* Label1 = new QLabel(TRS("Real Part"),this,"Label1");
   Label1->setGeometry(20,50,100,20);
   QLabel* Label2 = new QLabel(TRS("Imaginary Part"),this,"Label2");
   Label2->setGeometry(20,80,100,20);

   MandelCheck = new QCheckBox(TRS("Mandelbrot Type"), this, "MandelCheck");
   MandelCheck->setGeometry(20,110,150,20);
   MandelCheck->setChecked(Fractal->l[0]!=0);

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int BarnsleyCDlg::SetSpecificParams(void)
{
char* eptr;
double p0,p1;
long l0;

   p0 = strtod(ConstX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p1 = strtod(ConstY->text(), &eptr);
   if (*eptr!=0) return FALSE;

   l0 = 0;
   if (MandelCheck->isChecked()) 
      l0 = 1; 

   Fractal->p[0] = p0;
   Fractal->p[1] = p1;
   Fractal->l[0] = l0;
   return TRUE;
}

void BarnsleyCDlg::SetDefaults()
{
char Strg[32];

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   ConstX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(1));
   ConstY->setText(Strg);
   MandelCheck->setChecked(FALSE);
}