#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <qapp.h>
#include "basefrac.h"
#include "basefrac.moc"
#define TRS klocale->translate

// Liste mit den Fraktaltyp-Namen
char * FracNames[] = {"Mandelbrot", "Juliaset", "Lambda-Mandel", "Lambda-Julia",
            "1/Mandel", "Gingerbreadman", "Galaxy", "Hop", "Chip",
            "Mandel^N", "Julia^n", "Triple", "Henon", "Barnsley A",
            "Barnsley B", "Barnsley C", "Orbit 3", "Wankel", "Quadrat",
            "Lyapunov", "Star", "Lorenz", "Genetic", "Pheonix", "Pocken",
            "einsame Insel", "Mand4"};
int FracTypeCount = 27;                            // Anzahl der Typen
double DefPar[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; // Default-Parameter;
extern MandelApp* MApp;
// Basistyp fr smtliche Fraktale ...
BaseFract::BaseFract()
{
//fprintf(stdout, "BaseFract::BaseFract()\n");
}

void BaseFract::Init(int SizeX, int SizeY)
{
int i;

   //fprintf(stderr,"BaseFract::BaseFract(int, int)\n");
   SetDefaultCorners(&li,&re,&un,&ob);
   PicSizeX = SizeX;
   PicSizeY = SizeY;
   // dx = (re-li)/SizeX;
   // dy = (ob-un)/SizeY;
   ReInit();
   BailOut = 4.0;
   ConvMode = 0;     // for faster BailOut-Test
   ConvRadNorm=BailOut;
   ConvRadReal=0.0;
   ConvRadImag=0.0;
   ConvRadReal1=0.0;
   ConvRadImag1=0.0;

   //fprintf(stderr, "dx=%f   dy=%f\n",dx,dy);
   MaxIter = 250;

   OrbX=0;  // Orbit-Startpunkt
   OrbY=0;
   OrbZ=0;

   for (i=0;i<16;i++) p[i]=SetDefaultParams(i);
   for (i=0;i<16;i++) l[i]=0;    // Long-Parameters
   for (i=0;i<4;i++) s[i][0] = '\0';

   InsideMethod = INS_SOLIDCOLOR;
   OutsideMethod = OUT_ITER;
   InsideColor = 0;
   OutsideColor = 128;
   PotentialSlope = 500.0;
   strcpy(FileMask, "zm%0.4d");
   FileCount  = 0;
   ColorMode = COL_LINEAR;
   LogFirstColor = 0;
   LogShift = 0.05;
   AutoLogColor = FALSE;
}

void BaseFract::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2) {
   *x1 = -2.0;     // Gre des Fraktals
   *x2 =  2.0;
   *y1 = -1.5;
   *y2 =  1.5;
}

double BaseFract::SetDefaultParams(int Index) {

   if (Index<0) return 0.0;   // Range-check
   if (Index>=16) return 0.0;
   return DefPar[Index];   // Float-Params initialisieren
}

void BaseFract::ReInit()
{
   dx = (re-li)/PicSizeX;
   dy = (ob-un)/PicSizeY;
   ConvRadNorm=ConvRadReal=ConvRadImag=ConvRadReal1=ConvRadImag1=0.0;
   switch (ConvMode) {
      case 0 :  ConvRadNorm = BailOut; break;
      case 1 :  ConvRadReal = BailOut; break;
      case 2 :  ConvRadImag = BailOut; break;
      case 3 :  ConvRadReal = BailOut; ConvRadImag = BailOut; break;
      case 4 :  ConvRadReal1= BailOut; ConvRadImag1= BailOut; break;
      default : ConvRadNorm = BailOut; break;
   }
}

BaseFract::~BaseFract()
{
}

void BaseFract::SetSize(int SizeX, int SizeY)
{
   PicSizeX=SizeX;
   PicSizeY=SizeY;
   //dx = (re-li)/SizeX;
   //dy = (ob-un)/SizeY;
   ReInit();
}

void BaseFract::InitField(void)
{   // Initialisierung der Field-Typen vor jeder Bildberechnung
}   // hier leer

int BaseFract::CalcPoint(int x, int y)
{
double rx = li+dx*x;
double ry = un+dy*y;

   return (int)(100.0*(rx*rx-ry*ry*ry)); //Dummy
}

void BaseFract::InitOrbit(void)
// Initialisierung vor der ersten Orbit-Berechnung
{
   OrbX = 0;
   OrbY = 0;
   OrbZ = 0;
}
void BaseFract::CalcOrbit(void)
// Berechnung fr orbitale Tpen
{
// Orbit Typ1 Version a
double h;

   h = 1.0-OrbY+fabs(OrbX);
   OrbY = OrbX;
   OrbX = h;
}

void BaseFract::InitIAO(void)
// Initialisierung vor der ersten ImageAtOnce-Berechnung
{
}
void BaseFract::CalcIAO(void)
// Berechnung fr 'ImageAtOnce'
{
}

/*int BaseFract::ZoomIn(int x1, int y1, int x2, int y2)  
// Zoom auf diesen Ausschnitt;
{
double d, a_ratio;
int zx,zy;

   //fprintf(stdout,"x1=%d  y1=%d  x2=%d  y2=%d\n",x1,y1,x2,y2);
   //fprintf(stdout, "px=%d  py=%d\n",PicSizeX, PicSizeY);

   // Ecken richtig drehen
   zx = QMIN(x1,x2);
   zy = QMIN(y1,y2);
   x2=QMAX(x1,x2);
   y2=QMAX(y1,y2);
   x1=zx;   y1=zy;

   if (x2<=x1) return FALSE;    // forget it
   if (y2<=y1) return FALSE; 
//fprintf(stdout,"Zooming...\n");

   a_ratio = 1.0*PicSizeX/PicSizeY;
   y2  = (int)(y1 + (x2-x1)/a_ratio);
   d = (1.0*re-li)/PicSizeX;
   re = li + d*x2;
   li = li + d*x1;
   d = (1.0*ob-un)/PicSizeY;
   ob = un + d*y2;
   un = un + d*y1;
   //dx = (1.0*re-li)/PicSizeX;
   //dy = (1.0*ob-un)/PicSizeY;
   SetSize(PicSizeX, PicSizeY);
//   fprintf(stdout,"li=%f  re=%f  un=%f  ob=%f\n",li,re,un,ob);
   return TRUE;
}*/

int BaseFract::ZoomIn(int x1, int y1, int x2, int y2)
// zurckzoomen
{   int CenterX,CenterY;
    double Faktor;

    if (x1==x2) return FALSE;
    if (y1==y2) return FALSE;
    CenterX = (x1+x2) >> 1;
    CenterY = (y1+y2) >> 1;
    Faktor  = (1.0*PicSizeX) / fabs(x2-x1);
    ZoomIn(CenterX, CenterY, Faktor);
   return TRUE;
}

void BaseFract::ZoomIn(int x, int y, double Faktor)
// Zoomen auf diesen Punkt mit gegebenem Zoomfaktor
{
double cx,cy,dx,dy;

   // Kantenlngen
   dx = 0.5*(re-li)/Faktor;
   dy = 0.5*(ob-un)/Faktor;
   // Mittelpunkt
   cx = li + ((re-li))*x/PicSizeX;
   cy = un + ((ob-un))*y/PicSizeY;

   li = cx-dx;
   re = cx+dx;
   un = cy-dy;
   ob = cy+dy;
   SetSize(PicSizeX, PicSizeY);
}

void BaseFract::ZoomOut(int x1, int y1, int x2, int y2)
// zurckzoomen
{   int CenterX,CenterY;
    double Faktor;

    CenterX = (x1+x2) >> 1;
    CenterY = (y1+y2) >> 1;
    Faktor  = (1.0*PicSizeX) / fabs(x2-x1);
    ZoomOut(CenterX, CenterY, Faktor);
}

void BaseFract::ZoomOut(int x, int y, double Faktor)
// Zoomen auf diesen Punkt mit gegebenem Zoomfaktor
{
double cx,cy,dx,dy;

   // Kantenlngen
   dx = 0.5*(re-li)*Faktor;
   dy = 0.5*(ob-un)*Faktor;
   // Mittelpunkt
   cx = li + ((re-li)*x)/PicSizeX;
   cy = un + ((ob-un)*y)/PicSizeY;

   li = cx-dx;
   re = cx+dx;
   un = cy-dy;
   ob = cy+dy;
   SetSize(PicSizeX, PicSizeY);
}

int BaseFract::ConvTest1(double a, double b) {

   switch(ConvMode) {
      case 0 : return(a*a+b*b < BailOut);
      case 1 : return(a*a<BailOut);
      case 2 : return(b*b<BailOut);
      case 3 : return ( (a*a<BailOut) && (b*b<BailOut) );
      case 4 : return ( (a*a<BailOut) || (b*b<BailOut) );
   }
   return 0;
}

char* BaseFract::GetFracInfo(void)
{
   return "Fractal-Baseobject";
}

int BaseFract::SetupFrac(BaseFract* Frac) // type specific-setup
{
GenericDlg FDlg = GenericDlg(Frac, 0, "Fractal settings");

   if (FDlg.exec()) {
      return FDlg.SetSpecificParams();
   } else
      return FALSE;
}

void BaseFract::GetParamsBlock(struct MandDataType* MandData) 
// Parameterblock fllen
{
int Lauf;

   // Kopieren der Parameter
   (MandData->FractalType) = GetFracType();
   MandData->li = li;
   MandData->re = re;
   MandData->un = un;
   MandData->ob = ob;
   MandData->MaxIter = MaxIter;
   MandData->BailOut = BailOut;           // Konvergenzradius
   MandData->ConvMode = ConvMode;         // Konvergenztestmethode
   for (Lauf=0;Lauf<16;Lauf++)
      MandData->p[Lauf] = p[Lauf];        // Float-Parameter
   for (Lauf=0;Lauf<16;Lauf++)
      MandData->l[Lauf] = l[Lauf];        // Integer-Parameter
   MandData->InsideMode = InsideMethod;
   MandData->OutsideMode = OutsideMethod;
   MandData->InsideColor = InsideColor;
   MandData->OutsideColor = OutsideColor;
   MandData->PotentialSlope = PotentialSlope;
   for (Lauf=0;Lauf<4;Lauf++)
      strcpy(MandData->s[Lauf], s[Lauf]);
   strcpy(MandData->FileMask, FileMask);
   MandData->FileCount = FileCount;
   MandData->ColorMode = ColorMode;
   MandData->LogFirstColor = LogFirstColor;
   MandData->LogShift = LogShift;
   MandData->AutoLogColor = AutoLogColor;
}

void BaseFract::SetParamsBlock(struct MandDataType* MandData)
// Parameter lesen
{
int Lauf;
//fprintf(stdout,"Parameter lesen File=%s\n",DateiName);

   OutputParams();
   // Kopieren der Parameter
   li = MandData->li;
   re = MandData->re;
   un = MandData->un;
   ob = MandData->ob;
   MaxIter = MandData->MaxIter;
   BailOut = MandData->BailOut;           // Konvergenzradius
   ConvMode = MandData->ConvMode;         // Konvergenztestmethode
   for (Lauf=0;Lauf<16;Lauf++)
      p[Lauf] = MandData->p[Lauf];        // Float-Parameter
   for (Lauf=0;Lauf<16;Lauf++)
      l[Lauf] = MandData->l[Lauf];        // Integer-Parameter
   InsideMethod = MandData->InsideMode;
   OutsideMethod = MandData->OutsideMode;
   OutsideColor = MandData->OutsideColor;
   InsideColor = MandData->InsideColor;
   PotentialSlope = MandData->PotentialSlope;
   for (Lauf=0;Lauf<4;Lauf++)
      strcpy(s[Lauf], MandData->s[Lauf]);
   strcpy(FileMask,MandData->FileMask);
   FileCount = MandData->FileCount;
   ColorMode = MandData->ColorMode;
   LogFirstColor = MandData->LogFirstColor;
   LogShift = MandData->LogShift;
   AutoLogColor = MandData->AutoLogColor;

   ReInit();
}

void BaseFract::OutputParams() {
   fprintf(stdout,"\nParameter Debug Output\n");
//   fprintf(stdout,"Kennbytes: %c%c\n",MandData.Kenn1,MandData.Kenn2);
   fprintf(stdout,"Eckpunkte: li=%f,re=%f,un=%f,ob=%f\n",li,re,un,ob);
}

long BaseFract::Potential(long Iter, double a, double b) {
// Potenital berechnen
double pot,Radius;
double MaxColor = 256.0;

  // if (!PotentialColors) return Iter;

  //if (Iter == 0)
  //  return ((a*b<0)?  1:2);
    //return (fabs(a)*fabs(b))*MD.MaxIter/2 + 1;
    //return 0;

  pot = Iter + 2;
  Radius = sqrt(a*a + b*b);

  if (pot <= 0.0  ||  Radius < 1.0)
    pot = 0.0;
  else
    pot = log(Radius)/pow(2.0,pot);

  if (pot>0.0) {
    pot = sqrt(pot);
    pot = /*256.0*/ MaxColor - pot*PotentialSlope-1.0;
  } else
    pot = MaxColor - 1.0; // 256.0-1.0;

  if (pot < 1.0)
    pot = 1.0;
  if (pot >= MaxColor /*255*/ )
    pot = MaxColor-1; // 255

  return (long)(pot);
}


long BaseFract::FarbMult(long Iter, double a, double b) {
long i;
    if (fabs(b) < 0.01)
        return 0;
    else {
        i = (long)(Iter*a/b);
        if (i < 0) i=0;
        return i;
    }
}



long BaseFract::PostCompute(long Iter, double a, double b)
{
int ret;
double x;

   if (AutoLogFlag) return Iter;   // Berechnung der Startwerte fr Logcolormap

   ret=0;
   if (Iter >= MaxIter) {
      switch (InsideMethod) {
         case INS_SOLIDCOLOR : return InsideColor;break;  //ret = InsideColor; break;
         case INS_MAXITER    : ret = Iter; break;
         case INS_MULT       : ret = (long)(300.0*a*b); break;
         case INS_ALTITUDE   : ret = (long)(100.0*sqrt(a*a+b*b)); break;
         case INS_ADD        : ret = (long)(100.0*(a+b)); break;
         case INS_ATAN       : if (b==0.0) ret = 0; else
                               ret = (int)(100.0*atan(a/b)); break;
      }
   } else
   {
      switch (OutsideMethod) {
         case OUT_SOLIDCOLOR : ret = OutsideColor; break;
         case OUT_ITER       : ret = Iter; break;
         case OUT_POTENTIAL  : ret = Potential(Iter, a, b); break;
         case OUT_MULT       : ret = FarbMult(Iter, a, b); break;
         case OUT_ATAN       : if (b==0.0) ret = 0; else
                               ret = (int)(100.0*atan(a/b)); break;
         case OUT_ADD        : ret = (long)(100.0*(a+b)); break;
      }
   }
   switch (ColorMode) {
      case COL_LOG : if (MaxIter<=LogFirstColor) ret = 32; // FEHLER!!!!
                     else
                     if (ret-LogFirstColor < 0) ret = 0;
                     else {
                        if (LogShift<=0) LogShift=0.1;
                        x=(double)(ret-LogFirstColor)/(double)(MaxIter-LogFirstColor);
                        ret = (int)(256.0*(log(x+LogShift)-log(LogShift))/(log(1+LogShift)-log(LogShift)));
                        //ret = (int)(256.0*pow((double)(ret-LogFirstColor)/(double)(MaxIter-LogFirstColor),0.33333));
                     }
      default : break;
   }
   return ret;
}

int BaseFract::GetSpecialParamsBytes(void)
// Gre des type-specific Parameterblocks
// 0 = keine speziellen Parameter, sonst > 0
{ return 0; }

void* BaseFract::GetSpecialParamsBlock(void)
// Pointer auf Params
{
   return NULL;
}

void BaseFract::PutSpecialParamsBlock(void)
// Pointer auf Params zum Schreiben
{
   return;
}
// -----------------------------------------

GenericDlg::GenericDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
   Fractal = Frac;
   //setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

/*   QLabel* FormelLab = new QLabel(Fractal->GetFracInfo(), this, "FormelLab");
   FormelLab->setGeometry(10,20,280,95);
*/
   OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,100,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefaultsBtn");
   DefaultsBtn->setGeometry(110,100,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   CancelBtn = new QPushButton(TRS("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,100,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int GenericDlg::SetSpecificParams(void)
{
char* eptr;
double p[4];
int i;

   for (i=0; i<=ParNum; i++) {  // Parameter umwandeln
      p[i] = strtod(Par[i]->text(), &eptr);
      if (*eptr!=0) return FALSE;
   }
   for(i=0; i<=ParNum; i++)    // Paramezer speichern
      Fractal->p[i] = p[i];
      
   return TRUE;
}

void GenericDlg::SetDefaults()
{
char Strg[32];
int i;

   for(i=0; i<=ParNum; i++) {
      sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(i));
      Par[i]->setText(Strg);
   }
}

void GenericDlg::SetParameterNum(int Anz)
// Anzahl der Parameter setzen, die in dem Dialog angezeigt werden
{
   ParNum = Anz;
   if (ParNum>3) ParNum = 3;  // max. 4 Parameter

   setFixedSize(300,150+ParNum*30);
   OkBtn->setGeometry(20,100+ParNum*30,80,30);
   DefaultsBtn->setGeometry(110,100+ParNum*30,80,30);
   CancelBtn->setGeometry(200,100+ParNum*30,80,30);
}

void GenericDlg::SetHeader(const char* Text) {
   QLabel* Header = new QLabel(Text, this, "Header");
   Header->setGeometry(20,3,150,18);
}

void GenericDlg::SetParameterText(int ParNo, const char* Text) {
char Strg[20];

   if (ParNo>3) return;
   if (ParNo<0) return;

   sprintf(Strg, "Lab%d", ParNo);
   Lab[ParNo] = new QLabel(Text, this, Strg);
   Lab[ParNo]->setGeometry(20,20+(ParNo+1)*30,75,20);
   
   sprintf(Strg, "Par%d", ParNo);
   Par[ParNo] = new QLineEdit(this, Strg);
   sprintf(Strg, "%0.14f", Fractal->p[ParNo]);
   Par[ParNo]->setText(Strg);
   Par[ParNo]->setGeometry(100, 20+(ParNo+1)*30, 125, 22);
}
