#ifndef BASEFRAC_H
#define BASEFRAC_H
#include <qwidget.h>
#include <qlabel.h>
#include <qpushbt.h>
#include <qdialog.h>
#include <qlined.h>
#include <qfiledlg.h>
#include <qmsgbox.h> 
#include <qfile.h>
#include "fracglobals.h"
#include "mandelapp.h"
#include "parameter.h"

// Postcompute-Konstanten
#define INS_SOLIDCOLOR  0      // spezielle Farbe
#define INS_MAXITER 1          // variable Farbe
#define INS_MULT 2             // Multiplikation
#define INS_ALTITUDE 3         // Betrag |z|
#define INS_ADD 4              // Addition
#define INS_ATAN 5             // arcustangens
#define OUT_SOLIDCOLOR 1  
#define OUT_ITER 0
#define OUT_POTENTIAL 2
#define OUT_MULT 3
#define OUT_ATAN 4
#define OUT_ADD 5
// Konstanten fr die Farbgebung
#define COL_LINEAR 0           // lineare Farbverteilung
#define COL_LOG 1              // logarithmische Farbverteilung

#define ConvTest(a,b) \
  ( (a+b < ConvRadNorm) || (a<ConvRadReal) || (b<ConvRadImag) || \
  ( (a<ConvRadReal1) && (b<ConvRadImag1) ) )


class BaseFract
{
public:
   BaseFract();
   virtual ~BaseFract();
   virtual void Init(int SizeX, int SizeY);  // Main-Init
   virtual void ReInit();
   void SetSize(int SizeX, int SizeY);     // Gre setzen/ndern
   virtual void SetDefaultCorners(double *x1, double *x2, double *y1, double *y2);   // Eckpunkte setzen
   virtual double SetDefaultParams(int Index);    // Defaultparameter
   virtual int GetFracClass(void) { return FC_FIELD; }; // FracKlasse
   virtual void InitField(void);   // Init. der Field-Type vor jeder Berechnung
   virtual int CalcPoint(int x, int y);  // the important function.....
   virtual void InitOrbit(void); //Init vor der ersten Orbit-Berechnung
   virtual void CalcOrbit(void); //Orbit-Berechnung
   virtual void InitIAO(void); //Init vor ImageAtOnce
   virtual void CalcIAO(void); //ImageAtOnce-Berechnung
   int ZoomIn(int x1, int y1, int x2, int y2);  // Zoom auf diesen Ausschnitt;
   void ZoomIn(int x, int y, double Faktor);
   void ZoomOut(int x1, int y1, int x2, int y2); // Verkleinern
   void ZoomOut(int x, int y, double Faktor);
   virtual int GetFracType() { return FT_MANDEL; }; // Fraktaltyp
   virtual int ConvTest1(double a, double b);
   //obsolete
   virtual char* GetFracName(void) { return "Mandelbrot"; }  // Fraktalname
   virtual char* GetFracInfo(void);    // Informationen ber dieses Fraktal
   virtual int SetupFrac(BaseFract* Frac); // type specific-setup
   virtual void GetParamsBlock(struct MandDataType* MandData); // Parameter speichern
   virtual void SetParamsBlock(struct MandDataType* MandData); // Parameter laden
   virtual void OutputParams();   //Anzeige der Parameter fr Debugging
   long PostCompute(long Iter, double a, double b);
   virtual int GetSpecialParamsBytes(void);  // Gre des type-specific Parameterblocks
   virtual void* GetSpecialParamsBlock(void); // Pointer auf Params, lesen
   virtual void PutSpecialParamsBlock(void); // Params setzen
   
   double li,re,ob,un;       // Eckpunkte
   int MaxIter;              // Rechentiefe
   double BailOut;           // Konvergenzlimit
   int ConvMode;
   double OrbX,OrbY,OrbZ;    // Orbit-Variablen
   double p[16];             // Float-Params;
   long l[16];               // Long int -Params
   char s[4][64];            // String-Parameter

   // Postcompute-Variablen
   int InsideMethod,OutsideMethod; // Rechenmethoden Inside und Outside
   int InsideColor,OutsideColor;   // Feste Farben inside und outside
   double PotentialSlope;          // Potential
   char FileMask[32];        // Formattiereung doe Animationsdateien
   int  FileCount;           // Nummer des ersten Bildes
   int ColorMode;            // Farbgebung lin/log
   int LogFirstColor;        // Erster Iter bei log
   double LogShift;          // Verschiebung der Log-Funk. fr bessere Darst.
   int AutoLogColor;         // "LogFirstColor" automatisch ermitteln
   int AutoLogFlag;          // Flag fr Postcompute, nicht ausfhren,wenn True
private:
   long Potential(long Iter, double a, double b);
   long FarbMult(long Iter, double a, double b);

protected:
   double dx,dy;             // Schrittweiten
   int PicSizeX, PicSizeY;   // Bildgre
   double ConvRadNorm, ConvRadReal, ConvRadImag,ConvRadReal1,ConvRadImag1;
};

class GenericDlg : public QDialog
{
Q_OBJECT
public:
   GenericDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0);
   int SetSpecificParams(void);
   void SetHeader(const char* Text);   // berschrift (nicht Titel) 
   void SetParameterNum(int Anz);  // Anzahl der Parameter
   void SetParameterText(int Par, const char* Text);
signals:

private slots:
   void SetDefaults();  // Defaultparameter setzen

private:
   QPushButton *OkBtn, *DefaultsBtn, *CancelBtn;
   BaseFract* Fractal;   // Pointer to the Fractal
   QLineEdit *Par[4];    // Parameter
   QLabel* Lab[4];       // Textlabels
   int ParNum;           // Anzahl der Parameter 
};

#endif