// Fraktaltyp Chip
// p[0] = pixel.x
// p[1] = pixel.y
#include <stdio.h>
#include <stdlib.h>
#include "mymath.h"
#include "chip.h"
#include "chip.moc"
#define TRS klocale->translate

double ChipPar[3] = {-15, -19, 1};


void Chip::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void Chip::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -400;
   *x2 =  400;
   *y1 = -300;
   *y2 =  300;
}

double Chip::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 3) return 0.0;
   return ChipPar[Index];
}

void Chip::InitOrbit(void) {
   OrbX = p[0];
   OrbY = p[1];
}

void Chip::CalcOrbit(void)
// Berechnung fr orbitale Tpen
{
// Orbit
double h;

   h = OrbY-sign(OrbX)*cos(sqr(log(fabs(p[1]*OrbX-p[2]))))*atan(sqr(log(fabs(p[2]*OrbX-p[1]))));
   OrbY = p[0]-OrbX;
   OrbX = h;
}


char* Chip::GetFracInfo(void)
{
   return "Formula: x(0) = 0;   y(0) = 0\nloop {\n  x(n+1) = y(n) - sign(x(n)) * cos(sqr(ln(p1 * x(n) - p2))))\n              *atan(sqr(ln(fabs(p2 * x(n) - p1))))\n   y(n+1) = p0 - x(n);\n} };\n     until (x^2>Bail)";
}

int Chip::SetupFrac(BaseFract* Frac) // type specific-setup
{
ChipDlg CDlg = ChipDlg(Frac, 0, "Chip settings");

   if (CDlg.exec()) {
      return CDlg.SetSpecificParams();
   } else
      return FALSE;
}


// -----------------------------------------

ChipDlg::ChipDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[32];
   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Constant Parameters"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   ConstX = new QLineEdit(this, "ConstX");
   ConstX->setGeometry(100,50,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[0]);
   ConstX->setText(Strg);
   ConstY = new QLineEdit(this, "ConstY");
   ConstY->setGeometry(100,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[1]);
   ConstY->setText(Strg);
   ConstZ = new QLineEdit(this, "ConstZ");
   ConstZ->setGeometry(100,110,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[2]);
   ConstZ->setText(Strg);

   QLabel* Label1 = new QLabel("P1",this,"Label1");
   Label1->setGeometry(20,50,75,20);
   QLabel* Label2 = new QLabel("P2",this,"Label2");
   Label2->setGeometry(20,80,75,20);
   QLabel* Label3 = new QLabel("P3",this,"Label3");
   Label3->setGeometry(20,110,75,20);

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int ChipDlg::SetSpecificParams(void)
{
char* eptr;
double p0,p1,p2;

   p0 = strtod(ConstX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p1 = strtod(ConstY->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p2 = strtod(ConstZ->text(), &eptr);
   if (*eptr!=0) return FALSE;

   Fractal->p[0] = p0;
   Fractal->p[1] = p1;
   Fractal->p[2] = p2;
   return TRUE;
}

void ChipDlg::SetDefaults()
{
char Strg[32];

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   ConstX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(1));
   ConstY->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(2));
   ConstZ->setText(Strg);
}