#ifndef FRACGLOBALS_H
#define FRACGLOBALS_H

// Liste mit dem Fraktaltyp-Namen
extern char * FracNames[];
extern int FracTypeCount;                            // Anzahl der Typen

// Fraktalklassen
#define FC_FIELD   1    // Field-Fractal
#define FC_ORBIT   2    // Orbit-Fraktal
#define FC_IAO     3    // 'Image at Once'

// Fraktaltypen
#define FT_BASE   -1    // Fraktal-Basisobjekt
#define FT_MANDEL  0    // Mandelbrot
#define FT_JULIA   1    // Julia-Menge
#define FT_MLAMBDA 2    // Mandel-Lambda
#define FT_JLAMBDA 3    // Julia-Lambda
#define FT_INVMANDEL 4  // 1/Mandel
#define FT_GINGER  5    // Gingerbreadman (O)
#define FT_GALAXY  6    // Galaxy
#define FT_HOP     7    // Hop (O)
#define FT_CHIP    8    // Chip (O)
#define FT_MANDEL_N  9  // Mandel hoch n
#define FT_JULIA_N 10   // Julia^n
#define FT_TRIPLE  11   // Triple
#define FT_HENON   12   // Henon
#define FT_BARNSLEYJA  13   // Barnsley Julia Var A
#define FT_BARNSLEYB  14   // Barnsley Julia Var B
#define FT_BARNSLEYC  15   // Barnsley Julia Var C
#define FT_ORBIT3  16   // Orbit 3
#define FT_WANKEL  17   // Wankel
#define FT_QUADRAT 18   // Kwadrat
#define FT_LIAPI   19   // Lyapunov
#define FT_STAR    20   // STAR-like
#define FT_LORENZ  21   // Lorenz-Attraktor
#define FT_GENETIC 22   // Genetic aus ct
#define FT_PHEONIX 23
#define FT_POCKEN  24
#define FT_INSEL   25   // einsame Insel
#define FT_MAND4   26   // MandMix4

// Rechenmethode
#define CM_LINE    0    // Line by Line
#define CM_RECT    1    // Rechtecke

// Parameterstruktur eines Fraktales
struct MandDataType {
   short FractalType;              // Fraktaltyp, mu am Anfang der Struktur stehen !!!!!!!!!!!!!!!!!!!!
   double li,re,un,ob;             // Eckpunkte
   long MaxIter;                   // Iterationstiefe
   double BailOut;                 // Konvergenzradius
   short ConvMode;                 // Konvergenztestmethode
   double p[16];                   // Float-Parameter
   long l[16];                     // Integer-Parameter
   // neu V 0.2b
   double ZoomFactor;              // Zoomfaktor normal (Zoomin/ZoomOut)
   double ZoomFactorAnimation;     // Zoomfaktor Animationszoom
   long ZoomAnimationPics;         // # der Bilder pro Anim-Zoom
   long UpdateInterval;            // Zeilen zwischen zwei Bildupdates
   long UpdateBool;                // Anzeige whrend der Berechnung
   // neu V0.21
   short InsideMode, OutsideMode;  // Postcompute-Modi
   long InsideColor, OutsideColor; // Farben fr Solidcolor;
   double PotentialSlope;          // Potential
   short CalcMode;                 // Rechenmethode
   // neu V0.35
   char s[4][64];                  // String-Parameter
   // neu v0.4
   char FileMask[32];              // Dateiformattierer fr Animationsdateinamen
   long FileCount;                 // Zhler fr Dateien
   int ColorMode;                  // logarithmic Colormap
   int LogFirstColor;              // starting Color
   double LogShift;                // Function shift
   int AutoLogColor;               // Automatisch Startwert suchen
   int IterIncrement;              // Autom.Iteranpassung bei Animzoom
};

// Parameterheader zur Typerkennung
struct MandDataType1 {  //Diese Struktur dient zum Erkennes des Fraktaltypes
   char Kenn1, Kenn2;        // Kennbytes, 'MU','CM','ED'..
   char Version1,Version2;   // Prg-Version ...1=Vorkomma, ...2=Nachkomma
   short FractalType;
};

// Animationsdaten
// Anzahl der Bilder, Startbild ...
struct AnimFrameType {
   long NumPics, StartPic, EndPic, Increment;
   char Mask[128];
   long PosNeg;
   double Zoom;
   long DispBits;
};
// Eine Sttzstelle vom Spline
struct AnimCoordType {
   double X,Y;
};
// Struktur fr einen Spline (kommt 16 mal vor)
struct AnimSplineType {
   long Natuerlich;// parameter/natrlich
   double y1,yn;   // Randpunkte
   long Count;    // Anzahl der Sttzstellen
   AnimCoordType Points[50];
};

extern QDir MandelDir, ColorMapDir;

#endif