// Fraktaltyp Galaxy
// p[0] = pixel.x
// p[1] = pixel.y
#include <stdio.h>
#include <stdlib.h>
#include "galaxy.h"
#include "galaxy.moc"
#define TRS klocale->translate


double GalaxyPar[2] = {0.3, 0.3};

void Galaxy::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void Galaxy::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -1.25;
   *x2 =  1.25;
   *y1 = -1.0;
   *y2 =  1.0;
}

double Galaxy::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return GalaxyPar[Index];
}

int Galaxy::CalcPoint(int x, int y)
{
   double a,b,c;
   int iter;
   double rx,ry;

   rx = li+dx*x;
   ry = un+dy*y;
   a=rx;
   b=ry;
   iter=0;

   do {
      c = a*a-b*b;
      b = 2.0*a*b;
      a = c;
      a = a+a + p[0];
      b += p[1];
      iter++;
   } while ((iter<MaxIter) && ConvTest(a*a,b*b));
   return PostCompute(iter,a,b);
}

char* Galaxy::GetFracInfo(void)
{
   return "Formula: x(0)=Pixel;\nloop {\n   h = x(n)^2\n   x(n+1) = h + (a+P0,P1)\n} until (x(n+1)^2>Bail)";
}

int Galaxy::SetupFrac(BaseFract* Frac) // type specific-setup
{
GalaxyDlg GDlg = GalaxyDlg(Frac, 0, "Galaxy settings");

   if (GDlg.exec()) {
      return GDlg.SetSpecificParams();
   } else
      return FALSE;
}


// -----------------------------------------

GalaxyDlg::GalaxyDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[32];
   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Fractal-Constants"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   ConstX = new QLineEdit(this, "ConstX");
   ConstX->setGeometry(100,50,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[0]);
   ConstX->setText(Strg);
   ConstY = new QLineEdit(this, "ConstY");
   ConstY->setGeometry(100,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[1]);
   ConstY->setText(Strg);

   QLabel* Label1 = new QLabel("P0",this,"Label1");
   Label1->setGeometry(20,50,75,20);
   QLabel* Label2 = new QLabel("P1",this,"Label2");
   Label2->setGeometry(20,80,75,20);

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "CancelBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int GalaxyDlg::SetSpecificParams(void)
{
char* eptr;
double p0,p1;

   p0 = strtod(ConstX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p1 = strtod(ConstY->text(), &eptr);
   if (*eptr!=0) return FALSE;

   Fractal->p[0] = p0;
   Fractal->p[1] = p1;
   return TRUE;
}

void GalaxyDlg::SetDefaults()
{
char Strg[32];

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   ConstX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(1));
   ConstY->setText(Strg);
}