// Fraktaltyp Genetic
// l[0]  # Transformationen
// l[1]  Register
// l[2]  freigegebene Transformationen

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "genetic.h"
#include "genetic.moc"
#define TRS klocale->translate

#define TOTAL_TRANSFORMS 9  // Max-Anzahl der verschiedenen Transformationen
#define NUM_TRANSFORMS 100  // Max-Anzahl der Transformationen bei der Berechnung
#define NUM_REGISTERS 25    // Max-Anzahl der Rechenregister
#define TRANS_PROJ     0x0001  // Projektion
#define TRANS_SHIFT    0x0002  // Shift
#define TRANS_SHIFTB   0x0004  // Shiftback
#define TRANS_ROTATE   0x0008  // rotate
#define TRANS_ROTATE2  0x0010  // rotate2
#define TRANS_MULT     0x0020  // multiply
#define TRANS_SINE     0x0040  // sine
#define TRANS_COND     0x0080  // conditional
#define TRANS_COMPL    0x0100  // complex

typedef struct {double x,y,z;} vector;    // RGB-Farbvektor
typedef void transform(vector *source, vector *control, vector *dest);

transform *(transformlist[TOTAL_TRANSFORMS]);
vector reg[NUM_REGISTERS];  // Rechenregister

short int transformsequence[NUM_TRANSFORMS];   // Art der Transf. im iten Schritt
short int source[NUM_TRANSFORMS];              // Quellregister iter Schritt
short int dest[NUM_TRANSFORMS];                // Zielregister iter Schritt
short int control[NUM_TRANSFORMS];             // Kontrollregister iter Schritt

short int ParamArray[4*NUM_TRANSFORMS];        // Array fr die Parameterspeicherung

double GeneticPar[2] = {0, 0.0};
int TransNum;                            // Anzahl der Transformationen
int RegisterNum;                         // Anzahl der Register
int TransUsed;                           // benutzte Transfomationen

void projection(vector *source, vector *control, vector *dest) {
double scalarprod;

   if (!(TransUsed & TRANS_PROJ)) return;
   
   scalarprod = source->x*control->x + source->y*control->y + source->z*control->z;
   dest->x = scalarprod* source->x;
   dest->y = scalarprod* source->y;
   dest->z = scalarprod* source->z;
}

void shift(vector *source, vector *control, vector *dest) {
    // return;
   if (!(TransUsed & TRANS_SHIFT)) return;
   dest->x = source->x + control->x;
   if (dest->x >= 1.0) dest->x -= 1.0;
   dest->y = source->y + control->y;
   if (dest->y >= 1.0) dest->y -= 1.0;
   dest->z = source->z + control->z;
   if (dest->z >= 1.0) dest->z -= 1.0;
}

void shiftback(vector *source, vector *control, vector *dest) {
    // return;
   if (!(TransUsed & TRANS_SHIFTB)) return;
   dest->x = source->x - control->x;
   if (dest->x <= 0.0) dest->x += 1.0;
   dest->y = source->y - control->y;
   if (dest->y <= 0.0) dest->y += 1.0;
   dest->z = source->z - control->z;
   if (dest->z <= 0.0) dest->z += 1.0;
}

void rotate(vector *source, vector*, vector *dest) {
   if (!(TransUsed & TRANS_ROTATE)) return;
   dest->x = source->y;
   dest->y = source->z;
   dest->z = source->x;
}

void rotate2(vector *source, vector*, vector *dest) {
   if (!(TransUsed & TRANS_ROTATE2)) return;
   dest->x = source->z;
   dest->y = source->x;
   dest->z = source->y;
}

void multiply(vector *source, vector *control, vector *dest) {
   if (!(TransUsed & TRANS_MULT)) return;
   dest->x = source->x*control->x;
   dest->y = source->y*control->y;
   dest->z = source->z*control->z;
}

void sine(vector *source, vector *control, vector *dest) {
   if (!(TransUsed & TRANS_SINE)) return;
   dest->x = 0.5+0.5*sin(20.0*source->x*control->x);
   dest->y = 0.5+0.5*sin(20.0*source->y*control->y);
   dest->z = 0.5+0.5*sin(20.0*source->z*control->z);
}

void conditional(vector *source, vector *control, vector *dest) {
    // return;
   if (!(TransUsed & TRANS_COND)) return;
   if (control->x + control->y + control->z > 0.5) {
      dest->x = source->x;
      dest->y = source->y;
      dest->z = source->z;
   } else {
      dest->x = control->x;
      dest->y = control->y;
      dest->z = control->z;
   }
}

void complement(vector*, vector*, vector *dest) {
   if (!(TransUsed & TRANS_COMPL)) return;
   dest->x = 1.0-dest->x;
   dest->y = 1.0-dest->y;
   dest->z = 1.0-dest->z;
}

void InitTransforms(void) {
   transformlist[0] = &projection;
   transformlist[1] = &shift;
   transformlist[2] = &shiftback;
   transformlist[3] = &rotate;
   transformlist[4] = &rotate2;
   transformlist[5] = &multiply;
   transformlist[6] = &sine;
   transformlist[7] = &conditional;
   transformlist[8] = &complement;
}

void MakeTransSequ(void) {
int Lauf;

   srand(time(NULL));
   for (Lauf=0; Lauf < TransNum; Lauf++) {
      transformsequence[Lauf] = rand() % TOTAL_TRANSFORMS;
      source[Lauf]  = rand() % RegisterNum;
      dest[Lauf]    = rand() % RegisterNum;
      control[Lauf] = rand() % RegisterNum;
   }
   dest[TransNum-1]    = 0;
}

int Point(double x, double y) {
int i;

   // Rechenregister initialisieren
   for (i=0; i<RegisterNum; i++) {
      reg[i].x = x; //(x/SizeX);
      reg[i].y = y; //(y/SizeY);
      reg[i].z = ((double)i/(double)RegisterNum);
   }


   for (i=0; i<TransNum; i++) {
      transformlist[transformsequence[i]]( &(reg[source[i]]) ,
					   &(reg[dest[i]]),
					   &(reg[control[i]]));
   }

   // putpixel(x,y, (0.33*reg[0].x + 0.33*reg[0].y + 0.33*reg[0].z) * 256.0);
   return (long)((0.33*reg[0].x + 0.33*reg[0].y + 0.33*reg[0].z) * 256.0);

}


// -----------------

void Genetic::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);

   l[0] = 50;
   TransNum = 50;
   l[1] = 6;
   RegisterNum = 6;
   l[2] = 0x01FF;
   TransUsed = 0x01FF;  // verwendete Transformationen
   InitTransforms();    // verfgbare Transformationen initialisieren
   MakeTransSequ();     // Transformationssequenz initialisieren
   fprintf(stdout,"Genetic.Init\n");
}

void Genetic::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 =  0;     // Gre des Fraktals
   *x2 =  1;
   *y1 =  0;
   *y2 =  1;
}

double Genetic::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return GeneticPar[Index];
}

int Genetic::CalcPoint(int x, int y)
{
double rx = li+dx*x;
double ry = un+dy*y;

   return Point(rx,ry);
}

char* Genetic::GetFracInfo(void)
{
   return "Formula: x=c=Pixel;\n     loop {x=x^2+c};\n     until (x^2>Bail)";
}

int Genetic::SetupFrac(BaseFract* Frac) // type specific-setup
{
GeneticDlg MDlg = GeneticDlg(Frac, 0, "Genetic settings");
int h;
   if (MDlg.exec()) {
      h = MDlg.SetSpecificParams();
      TransNum = l[0];
      RegisterNum = l[1];
      TransUsed = l[2];
      InitTransforms();
      MakeTransSequ();
//      fprintf(stdout,"TransNum=%d,  RegNum=%d,  TransUsed=%X\n",l[0],l[1],l[2]);
      return h;
   } else
      return FALSE;
}

int Genetic::GetSpecialParamsBytes(void)
{
   return sizeof(ParamArray);
}

void* Genetic::GetSpecialParamsBlock(void)
// Pointer auf Params, lesen
{
int i;

   for (i=0; i<NUM_TRANSFORMS;i++) {
      ParamArray[i] = transformsequence[i];
      ParamArray[i+NUM_TRANSFORMS] = source[i];
      ParamArray[i+2*NUM_TRANSFORMS] = dest[i];
      ParamArray[i+3*NUM_TRANSFORMS] = control[i];
   }
   //for (i=0; i<(4*NUM_TRANSFORMS); i++)
   //   ParamArray[i]=i;
   return &ParamArray;
}

void Genetic::PutSpecialParamsBlock(void)
{
int i;
fprintf(stdout,"vor putspec...\n");
   for (i=0;i<NUM_TRANSFORMS;i++) {
      transformsequence[i] = ParamArray[i];
      source[i] = ParamArray[i+NUM_TRANSFORMS];
      dest[i] = ParamArray[i+2*NUM_TRANSFORMS];
      control[i] = ParamArray[i+3*NUM_TRANSFORMS];
   }
fprintf(stdout,"nach putspec...\n");
}


// -----------------------------------------

GeneticDlg::GeneticDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
int i;
char* Names[] = {"Projektion","Shift","ShiftBack","Rotate","Rotate2","Multiply","Sine","Conditional","Complement"};

   Fractal = Frac;
   setFixedSize(350,250);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Genetic Parameters"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   TransformSpin = new KNumericSpinBox(this, "TransformSpin");
   TransformSpin->setGeometry(130,50,50,20);
   TransformSpin->setRange(5,50);
   TransformSpin->setValue(Fractal->l[0]);
   TransformSpin->setEditable(TRUE);

   RegisterSpin = new KNumericSpinBox(this, "RegisterSpin");
   RegisterSpin->setGeometry(130,80,50,20);
   RegisterSpin->setRange(3,25);
   RegisterSpin->setValue(Fractal->l[1]);
   RegisterSpin->setEditable(TRUE);
   
   QLabel *Label1 = new QLabel(TRS("#Transforms"), this, "Label1");
   Label1->setGeometry(20,50,105,20);
   QLabel *Label2 = new QLabel(TRS("#Registers"), this, "Label2");
   Label2->setGeometry(20,80,105,20);
   
   for (i=0;i<9;i++) {
      TransCheck[i] = new QCheckBox(Names[i], this, Names[i]);
      TransCheck[i]->setGeometry(200,10+19*i,150,20);
      TransCheck[i]->setChecked(Fractal->l[2]&(1<<i));
   }


   QPushButton* EdgesBtn = new QPushButton(TRS("No Edges"), this, "EdgesBtn");
   EdgesBtn->setGeometry(20,140,100,30);
   connect(EdgesBtn,SIGNAL(clicked()), this, SLOT(NoEdges()) );

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,200,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,200,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "CancelBtn");
   CancelBtn->setGeometry(200,200,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int GeneticDlg::SetSpecificParams(void)
{
int i,h;

   Fractal->l[0] = TransformSpin->getValue();
   Fractal->l[1] = RegisterSpin->getValue();
   h=0;
   for(i=0;i<9;i++) {
      if (TransCheck[i]->isChecked())
         h += 1<<i;
   }
   Fractal->l[2]=h;
   return TRUE;
}

void GeneticDlg::SetDefaults()
{
int i;

   TransformSpin->setValue(50);
   RegisterSpin->setValue(6);
   for (i=0;i<9;i++)
      TransCheck[i]->setChecked(TRUE);
}

void GeneticDlg::NoEdges()
{
   TransCheck[1]->setChecked(FALSE);  // Shift
   TransCheck[2]->setChecked(FALSE);  // Shiftback
   TransCheck[7]->setChecked(FALSE);  // Conditional
}