// Fraktaltyp Ginger
// p[0] = pixel.x
// p[1] = pixel.y
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "ginger.h"
#include "ginger.moc"
#define TRS klocale->translate

double GingerPar[2] = {-0.1, 0};

void Ginger::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   l[0] = 0;
   SetSize(SizeX, SizeY);
}

void Ginger::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -6;
   *x2 =  9;
   *y1 = -6;
   *y2 =  9;
}

double Ginger::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return GingerPar[Index];
}

void Ginger::InitOrbit(void) {
   OrbX = p[0];
   OrbY = p[1];
}

void Ginger::CalcOrbit(void)
// Berechnung fr orbitale Tpen
{
// Orbit Typ1 Version a
double h;

   if (l[0])
      h = 1.0-OrbY-fabs(OrbX);   // Variante
   else
      h = 1.0-OrbY+fabs(OrbX);   // Orginal
   OrbY = OrbX;
   OrbX = h;
}


char* Ginger::GetFracInfo(void)
{
   return "Formula:\nx0=(PX,PY);\nloop {\n   x(n+1) = 1 - y(n) + |x(n)|\n   y(n+1) = x(n)\n} until (loops>100*MaxIter)";
}

int Ginger::SetupFrac(BaseFract* Frac) // type specific-setup
{
GingerDlg GDlg = GingerDlg(Frac, 0, "Gingerbreadman settings");

   if (GDlg.exec()) {
      return GDlg.SetSpecificParams();
   } else
      return FALSE;
}


// -----------------------------------------

GingerDlg::GingerDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[32];
   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Starting Point X(0)"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   ConstX = new QLineEdit(this, "ConstX");
   ConstX->setGeometry(100,50,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[0]);
   ConstX->setText(Strg);
   ConstY = new QLineEdit(this, "ConstY");
   ConstY->setGeometry(100,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[1]);
   ConstY->setText(Strg);
   VariantCheck = new QCheckBox(TRS("Variant"), this, "VariantCheck");
   VariantCheck->setGeometry(70,110,100,20);
   VariantCheck->setChecked(Fractal->l[0]);

   QLabel* Label1 = new QLabel("PX",this,"Label1");
   Label1->setGeometry(20,50,75,20);
   QLabel* Label2 = new QLabel("PY",this,"Label2");
   Label2->setGeometry(20,80,75,20);

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "CancelBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int GingerDlg::SetSpecificParams(void)
{
char* eptr;
double p0,p1;

   p0 = strtod(ConstX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p1 = strtod(ConstY->text(), &eptr);
   if (*eptr!=0) return FALSE;

   Fractal->l[0] = VariantCheck->isChecked();
   
   Fractal->p[0] = p0;
   Fractal->p[1] = p1;
   return TRUE;
}

void GingerDlg::SetDefaults()
{
char Strg[32];

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   ConstX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(1));
   ConstY->setText(Strg);
   VariantCheck->setChecked(FALSE);
}