// Orbitaltyp Henon
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mymath.h"
#include "henon.h"
#include "henon.moc"
#define TRS klocale->translate

double HenonPar[4] = {0,0, 1.4, 0.3};

void Henon::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void Henon::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -1.5;
   *x2 =  1.5;
   *y1 = -0.5;
   *y2 =  0.5;
}

double Henon::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 4) return 0.0;
   return HenonPar[Index];
}

void Henon::CalcOrbit(void)
// Berechnung fr orbitale Tpen
{
double h;

   h = 1.0+OrbY-p[2]*OrbX*OrbX;
   OrbY = p[3]*OrbX;
   OrbX = h;
}


char* Henon::GetFracInfo(void)
{
   return "Orbit in two dimensions\nFormula:\nX(0)=p0  Y(0)=p1\n\nX(n+1)=1+Y(n)-A*X(n)^2\nY(n+1)=B*X(n)";
}

int Henon::SetupFrac(BaseFract* Frac) // type specific-setup
{
HenonDlg GDlg = HenonDlg(Frac, 0, "Henon settings");

   if (GDlg.exec()) {
      return GDlg.SetSpecificParams();
   } else
      return FALSE;
}


// -----------------------------------------

HenonDlg::HenonDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[32];
   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Parameters"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   ConstX = new QLineEdit(this, "ConstX");
   ConstX->setGeometry(100,50,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[2]);
   ConstX->setText(Strg);
   ConstY = new QLineEdit(this, "ConstY");
   ConstY->setGeometry(100,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[3]);
   ConstY->setText(Strg);

   QLabel* Label1 = new QLabel("A",this,"Label1");
   Label1->setGeometry(20,50,75,20);
   QLabel* Label2 = new QLabel("B",this,"Label2");
   Label2->setGeometry(20,80,75,20);

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int HenonDlg::SetSpecificParams(void)
{
char* eptr;
double p2,p3;

   p2 = strtod(ConstX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p3 = strtod(ConstY->text(), &eptr);
   if (*eptr!=0) return FALSE;

   Fractal->p[2] = p2;
   Fractal->p[3] = p3;
   return TRUE;
}

void HenonDlg::SetDefaults()
{
char Strg[32];

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(2));
   ConstX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(3));
   ConstY->setText(Strg);
}