// Fraktaltyp Insel
// Parameter
// l0 : Inselnummer
// l1 : Wasserstand
// l2 : Inselfaktor (Skalierung)
// l3 : Rotationsflags  (Bit 0 LiRe  Bit1 vohi)
#define InselNr l[0]
#define WasserStand l[1]
#define InselFaktor l[2]
#define Flags l[3]

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mymath.h"
#include "insel.h"
#include "insel.moc"
#define TRS klocale->translate


double InselPar[2] = {1, 0.0};

extern KMandel *kmand; 

void Insel::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
   InselNr = 5;
   WasserStand = 60;
   InselFaktor = 256;
   Flags = 3;
   ReInit();
}

void Insel::ReInit(void){
   fprintf(stdout,"Insel.reinit\n");
   memset(Feld1, 0, sizeof(Feld1));
   memset(Feld2, 0, sizeof(Feld2));
} 

void Insel::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 =  0;     // Gre des Fraktals
   *x2 =  1;
   *y1 =  0;
   *y2 =  1;
}

double Insel::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return InselPar[Index];
}


void Insel::InitIAO(void)
{
   BerechneInsel();
}
void Insel::CalcIAO(void)
{

//   kmand->Client->FillRect(10,10,100,100,32);
//   kmand->Client->FillRect(0,0,600,600,64);
//   kmand->Client->FillQuad(10,10, 100,20, 120,100, 30,110, 64);
   InselDarstellen();
   return;
}

char* Insel::GetFracInfo(void)
{
   return 
"Reif fr die Insel\n\
aus C't 3.91\n\n\
Am besten mit 8bpp und Farbtabelle 'Insel.map' betrachten.";
}

int Insel::SetupFrac(BaseFract* Frac) // type specific-setup
{
InselDlg MDlg = InselDlg(Frac, 0, "Insel settings");
int h;
   if (MDlg.exec()) {
      h = MDlg.SetSpecificParams();
      ReInit();
      return h;
   } else
      return FALSE;
}

void Insel::BerechneInsel(void) {
int max, zuf, i,n,m,n2,m2;

   max=2;
   zuf=InselFaktor;
   srand(InselNr);
   for (i=1; i<=6; i++) {
      // Alle Punkte um Zufallswerte erhhen
      for (n=1; n<=max; n++) {
         for (m=1; m<=max; m++)
            Feld1[n][m]=Feld1[n][m]+(rand()%zuf);
      }
      // den Rand auf Nullhhe halten blablabla....
      for (n=1;n<=max;n++) {
         Feld1[n][1]=0;
         Feld1[n][max]=0;
         Feld1[1][n]=0;
         Feld1[max][n]=0;
      }
      //kopieren
      for (n=1; n<=max; n++) {
         for (m=1; m<=max; m++) {
            n2 = (n-1)*2+1;
            m2 = (m-1)*2+1;
            Feld2[n2][m2]=Feld1[n][m];
            Feld2[n2][m2+2]=Feld1[n][m+1];
            Feld2[n2+2][m2]=Feld1[n+1][m];
            Feld2[n2+2][m2+2]=Feld1[n+1][m+1];
            Feld2[n2+1][m2+1]=(Feld1[n][m]+Feld1[n+1][m+1]) >> 1;
            Feld2[n2+1][m2+2]=(Feld1[n][m+1]+Feld1[n+1][m+1]) >> 1;
            Feld2[n2+2][m2+1]=(Feld1[n+1][m]+Feld1[n+1][m+1]) >> 1;
            Feld2[n2+1][m2]=(Feld1[n][m]+Feld1[n+1][m]) >> 1;
            Feld2[n2][m2+1]=(Feld1[n][m]+Feld1[n][m+1]) >> 1;
         }
      }

      Feld1=Feld2;
      max=max+max;
      zuf=zuf>>1;
   }
   
   for (m=0;m<129;m++)   // Mirror
   for (n=0;n<129;n++) {
      if (Flags & 1) m2=128-m; else m2=m;
      if (Flags & 2) n2=128-n; else n2=n;
      Feld2[m][n]=Feld1[m2][n2];
   }
   Feld1=Feld2;
}

long Insel::CalcFarbe(int z1, int z2, int z3, int z4) {
int n1,n2;

   n1=((z3+z4) >> 1) - ((z1+z2)>>1);
   n2=((z2+z3) >> 1) - ((z1+z4)>>1);
   if (n1>5) n1=5;
   if (n1<-5) n1=-5;
   if (n2>5) n2=5;
   if (n2<-5) n2=-5;
   return (long)(32+(n2+n1)*3.1);
}

void Insel::InselDarstellen(void) {
int yp,xp,w;
int h1,h2,h3,h4,t1,t2,t3,t4;
long y1,y2,y3,y4,x1,x2,x3,x4;
int t,f,fa,hd;
double gfx,gfy;

   gfx = PicSizeX/320.0;
   gfy = PicSizeY/200.0;
   if (gfx<0.1) gfx=0.1;
   if (gfy<0.1) gfy=0.1;
   w=WasserStand;
   for (yp=127; yp>=1; yp--) {
      //fprintf(stdout,"Darstellen %d\n",yp);
      for (xp=127; xp>=1; xp--) {
         h1=Feld1[xp][yp];
         t1=w-h1;
         if (h1<w) h1=w;
         x1=xp*2+yp-25;
         y1=200-(yp+(h1>>1));
         
         h2=Feld1[xp][yp+1];
         t2=w-h2;
         if (h2<w) h2=w;
         x2=xp*2+yp+1-25;
         y2=200-(yp+1+(h2>>1));
         
         h3=Feld1[xp+1][yp+1];
         t3=w-h3;
         if (h3<w) h3=w;
         x3=(xp+1)*2+yp+1-25;
         y3=200-(yp+1+(h3>>1));
         
         h4=Feld1[xp+1][yp];
         t4=w-h4;
         if (h4<w) h4=w;
         x4=(xp+1)*2+yp-25;
         y4=200-(yp+(h4>>1));
         
//         printf("nach gen %d\n",xp);
         if ((h1==w) && (h2==w) && (h3==w) && (h4==w)) {
            t = t1+t2+t3+t4;
            if (t<40) {
               kmand->Client->FillQuad((int)(x1*gfx),(int)(y1*gfy),(int)(x2*gfx),(int)(y2*gfy),(int)(x3*gfx),(int)(y3*gfy),(int)(x4*gfx),(int)(y4*gfy),  14);
            } else
            {
               t=t-40;
               f=127-(t/6);
               if (f<64) f=64;
               kmand->Client->FillQuad((int)(x1*gfx),(int)(y1*gfy),(int)(x2*gfx),(int)(y2*gfy),(int)(x3*gfx),(int)(y3*gfy),(int)(x4*gfx),(int)(y4*gfy),  f);
            }
         } else
         {
            hd = h1+h2+h3+h4;
            fa = CalcFarbe(h1,h2,h3,h4);
            if (hd>550)
               fa=(fa/4)+40;
            else {
               if (hd<400)
                  fa=fa+128;
               else
                  fa=fa+192;
            }
               kmand->Client->FillQuad((int)(x1*gfx),(int)(y1*gfy),(int)(x2*gfx),(int)(y2*gfy),(int)(x3*gfx),(int)(y3*gfy),(int)(x4*gfx),(int)(y4*gfy),  fa);
         }
      }
      qApp->processEvents();
   }
}

// -----------------------------------------

InselDlg::InselDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{

   Fractal = Frac;
   setFixedSize(300,250);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Island Parameters"), this, "Label0");
   Label0->setGeometry(30,5,150,20);

   InselNrSpin = new KNumericSpinBox(this, "InselNrSpin");
   InselNrSpin->setGeometry(120,35,50,20);
   InselNrSpin->setRange(0,32767);
   InselNrSpin->setValue(Fractal->l[0]);
   InselNrSpin->setEditable(TRUE);

   WasserSpin = new KNumericSpinBox(this, "WasserSpin");
   WasserSpin->setGeometry(120,65,50,20);
   WasserSpin->setRange(10,75);
   WasserSpin->setValue(Fractal->l[1]);
   WasserSpin->setEditable(TRUE);

   HoeheSpin = new KNumericSpinBox(this, "HoeheSpin");
   HoeheSpin->setGeometry(120,95,50,20);
   HoeheSpin->setRange(80,260);
   HoeheSpin->setValue(Fractal->l[2]);
   HoeheSpin->setEditable(TRUE);

   HorCheck = new QCheckBox(TRS("Mirror horizontal"), this, "HorCheck");
   HorCheck->setGeometry(20,135,150,18);
   HorCheck->setChecked(Fractal->Flags & 1);
   
   VertCheck = new QCheckBox(TRS("Mirror vertical"), this, "VertCheck");
   VertCheck->setGeometry(20,155,150,18);
   VertCheck->setChecked(Fractal->Flags & 2);
   
   QLabel *Label1 = new QLabel(TRS("Islandnumber:"), this, "Label1");
   Label1->setGeometry(20,35,90,20);
   QLabel *Label2 = new QLabel(TRS("Waterlevel:"), this, "Label2");
   Label2->setGeometry(20,65,90,20);
   QLabel *Label3 = new QLabel(TRS("Islandheight:"), this, "Label3");
   Label3->setGeometry(20,95,90,20);
   

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,200,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,200,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,200,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int InselDlg::SetSpecificParams(void)
{

   Fractal->l[0] = InselNrSpin->getValue();
   Fractal->l[1] = WasserSpin->getValue();
   Fractal->l[2] = HoeheSpin->getValue();
   Fractal->l[3] = 0;
   if (HorCheck->isChecked())
      Fractal->l[3] |= 1;
   if (VertCheck->isChecked())
      Fractal->l[3] |= 2;
   return TRUE;
}

void InselDlg::SetDefaults()
{
   InselNrSpin->setValue(5);
   WasserSpin->setValue(60);
   HoeheSpin->setValue(256);
   HorCheck->setChecked(1);
   VertCheck->setChecked(1);
}

