// Fraktaltyp Julia-Menge
// p[0] = pixel.x
// p[1] = pixel.y
#include <stdio.h>
#include <stdlib.h>
#include "invmandel.h"
#include "invmandel.moc"
#define TRS klocale->translate

double InvMandPar[2] = {1, 0};

void InvMand::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void InvMand::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -1.5;
   *x2 =  4;
   *y1 = -2.0;
   *y2 =  2.0;
}

double InvMand::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return InvMandPar[Index];
}

int InvMand::CalcPoint(int x, int y)
{
   int iter;

struct komplex z,h,init;

  iter = 0;
  z.re=li+dx*x; 
  z.im=un+dy*y;
  init.re=p[0];
  init.im=p[1];
  z = cdiv(init,z);
  h = z;
  do {
    z = cmul(z,z);
    z = cadd(z,h);
    iter++;
  } while ( (iter<MaxIter) && ConvTest(z.re*z.re,z.im*z.im) );


   return PostCompute(iter,z.re,z.im);
}

char* InvMand::GetFracInfo(void)
{
   return "1/Mandel";
}

int InvMand::SetupFrac(BaseFract* Frac) // type specific-setup
{
InvMandDlg IVMDlg = InvMandDlg(Frac, 0, "1/Mandel settings");

   if (IVMDlg.exec()) {
      return IVMDlg.SetSpecificParams();
   } else
      return FALSE;
}


// -----------------------------------------

InvMandDlg::InvMandDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[32];
   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Initial value of X(0)"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   ConstX = new QLineEdit(this, "ConstX");
   ConstX->setGeometry(100,50,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[0]);
   ConstX->setText(Strg);
   ConstY = new QLineEdit(this, "ConstY");
   ConstY->setGeometry(100,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[1]);
   ConstY->setText(Strg);

   QLabel* Label1 = new QLabel(TRS("InitX"),this,"Label1");
   Label1->setGeometry(20,50,75,20);
   QLabel* Label2 = new QLabel(TRS("InitY"),this,"Label2");
   Label2->setGeometry(20,80,75,20);

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int InvMandDlg::SetSpecificParams(void)
{
char* eptr;
double p0,p1;

   p0 = strtod(ConstX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p1 = strtod(ConstY->text(), &eptr);
   if (*eptr!=0) return FALSE;

   Fractal->p[0] = p0;
   Fractal->p[1] = p1;
   return TRUE;
}

void InvMandDlg::SetDefaults()
{
char Strg[32];

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   ConstX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(1));
   ConstY->setText(Strg);
}