#ifndef MANDEL_M
#define MANDEL_M
#include <stdio.h>
#include <qapp.h>
#include <qpushbt.h>
#include <qfont.h>
#include <qlabel.h>
#include <qcolor.h>
#include <qmenubar.h>
#include <qpopmenu.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qwindow.h>
#include <kfiledialog.h>
#include <qfile.h>
#include <kapp.h>
#include <ktopwidget.h>
#include <kkeyconf.h>



#include <kstatusbar.h>
#include <kiconloader.h> 
#include <ktoolbar.h>
#include "fracglobals.h"
#include "basefrac.h"

#define ZOOM_NONE 0       // kein Zooming
#define ZOOM_NORM 1       // normales Zooming
#define ZOOM_CLICK 2      // Point & Click
#define ZOOM_MOVE 3       // Rectangle moving, Fractint-compatible
#define ZOOM_ANIMATION 4  // Animation-Zoom, automatic calculation

// Konstanten fr die Statuszeile
#define SZ_TYPE  0        // Feld Fraktaltyp
#define SZ_PROGRESS 1     // Fortschrittsanzeige
#define SZ_ZOOM 2         // Zoommodus

/**
* DrawWidget : Klasse zum Darstellen des Bildes
*/
class DrawWidget : public QWindow
{
   Q_OBJECT
public:
   /**
   * Construktor
   */
   DrawWidget( QWidget* parent=0, const char* name=0, WFlags f=0 );
   /**
   * Setzt die Gre Des Bildes in Pixeln.
   * @param SizeX neue Breite des Bildes
   * @param SizeY neue Hhe
   */
   void SetPicSize(int SizeX, int SizeY);  // Gre setzen

   /**
   * Ermittelt die horizontale Bildgre
   * @return Breite des Bildes 
   */
   int GetSizeX(void) {return PicSizeX; }        // x-Gre lesen

   /**
   * Ermittelt die vertikale Bildgre in Pixeln
   * @return Hhe des Bildes 
   */
   int GetSizeY(void) {return PicSizeY; }        // y-Gre lesen

   /**
   * Bricht die gerade laufende Berechnung ab. Die Variable Berechnung wird
   * auf FALSE gesetzt. Die Rechenroutine des Fraktals mu diese Variable
   * abfragen und dann die Berechnung beenden. Es kann daher eine
   * kurze Zeit dauern, bis die Berechnung wirklich beendet wurde.
   */
   void AbortCalc(void);           // Berechnung abbrechen

   /**
   * Wechsel des Fraktaltypes.
   * Der neue Fraktaltyp wird mit den Defaultwerten initialisiert und dann
   * die Berechnung gestartet. Eine evtl. laufende Berechnung wird vorher
   * mit AbortCalc beendet.
   * @see #AbortCalc
   */
   void NewFractal(int FracType);  // neues Fraktal(-typ) berechnen

   /**
   * Ldt eine Farbtabelle fr das aktuelle Bild. Wenn die Farbtiede mehr als
   * 8 Bit ist, wird keine Farbtabelle benutzt.
   * @param MapName  Dateiname der Farbtabelle
   */
   int LoadColorMap(char* MapName);
   
   /**
   * Speichern der aktuellen Farbtabelle. Vorher wird in einem Dateidialog der
   * Dateiname erfragt.
   * @return FALSE,wenn Datei nicht gespeichert werden konnte, sonst TRUE.
   */
   int SaveColorMap(void);         // Farbpalette speichern

   /**
   * Farbtabelle auswhlen und laden. In einem Dateidialog wird die Tabelle
   * gewhlt, dann mit LoadColorMap() geladen.
   * @see #LoadColorMap
   */
   int SelectColorMap(void);       // Palette laden
   void ChangePicParams(int sx, int sy, int cdeep); // Bildparameter ndern
   void EditColorMap(void);        // Palette editieren
   void CalcFractal(void);         // berechnen, fr alle Typen
   void CalcMandel(void);          // berechnen, nur Field-Types
   void CalcOrbit(void);           //     "    , nur Orbits
   void CalcImageAtOnce(void);     //     "    , nur Orbits
   void InitPalette(void);         // Farbpalette initialisieren
   void LoadPic(char* DateiName);  // Image laden
   void GetParamsBlock(struct MandDataType* MandData); // Daten fr speichern
   void SetParamsBlock(struct MandDataType* MandData); // Daten aus Datei setzen
   void PutPixel(int x, int y, long Iter);  
   long GetPixel(int x, int y);  
   void PicToColorMap(void);       // Farbpalette aus dem gelesenen Bild 
   // Hilfsroutinen (werden zZ. von 'Insel' bentigt
   void FillRect(int X, int Y, int B, int H, long Col);
   void FillQuad(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4, int Color);
   void Taste(QKeyEvent* kev);
   int SaveImage(char* Dateiname);
   void WriteParams(QString* DN);      // Faktalparameter speichern
   
   BaseFract * Fractal;            // das Fraktal
   int ColorDeep;                  // Farbtiefe 1,8,32
   unsigned long NumColors;        // Anzahl der Farben
   double ZoomFactor;              // Zoomfaktor normal (Zoomin/ZoomOut)
   double ZoomFactorAnimation;     // Zoomfaktor Animationszoom
   int ZoomAnimationPics;          // # der Bilder pro Anim-Zoom
   int IterIncrement;              // Erhhung des Maxiter
   int SaveColorMapBool;           // Farbpalette mit dem Parameterfile speichern
   int UpdateInterval;             // Lines zwischen zwei Screenupdates
   int UpdateLevel;                // UI fr Rechenmode "Rectangle"
   int UpdateBool;                 // Anzeige whrend der Berechnung (J/N)
   int HiColorMode;                // Farbgebung im Hicolor-Mode
   int CalcMethod;                 // Rechenmethode
   int ZoomRectVisible;            // Zoombox sichtbar (!=0)
   
   int Berechnung;           // TRUE=Berechnung luft, FALSE keine Berechnung
   int ImageReady;           // Das Bild ist fertig und kann direkt dargestellt werden
   int LoadImage;
   
   unsigned long ColorMap[256];    // Farbpalette
protected:
   void paintEvent( QPaintEvent * );   
   void resizeEvent( QResizeEvent * );
   void mousePressEvent (QMouseEvent*);
   void mouseReleaseEvent (QMouseEvent*);
   void mouseDoubleClickEvent (QMouseEvent*);
   void mouseMoveEvent (QMouseEvent*);
//   void	keyPressEvent(QKeyEvent* kev);
signals:
   void FertigSig();         // Berechnung ist fertig
//   void Start();             // Berechnung starten
public slots:
   void MenuZoomIn();
   void MenuZoomOut();
   void MenuZoomMode();      // Umschaltung des Zoommode
   void MenuSavePic();       // Bild speichern
   void Fertig();            // Aktion bei Berechnung fertig
private:
   QImage * Image;           // Puffer fr berechnetes Bild
   QPixmap * Pix;
   QPainter * Paint;
   int PicSizeX,PicSizeY;    // Gre des Bildes in Pixeln
   int Resized;              // Die Bildgre wurde gendert, neu berechnen...

   int ZoomMode;             // Modus fr Zooming
   int ZoomX1,ZoomY1;        // Koordinaten des 1. Punktes
   int ZoomX2,ZoomY2;        // Koordinaten des 2. Punktes
   int zxa,zya,bra,hoa;      // zum Lschen des Rechtecks
   int ZoomRectX,ZoomRectY;  // Gre des Zoomrect (im Drag-Mode)
   long PixCount,ModVal;     // # der Pickel, fr Status
   long PixMax;              // Gesamtanzahl der Pickel
   
   
   int miny,maxy;            // Hilfsvars. fr Fillquad
   int tst[1001][2];          // fr FillQuad
   
   void PaletteInterpolate(int Start, int Ende); // Farbverlauf
   void CreatePalette(void);
   int TestRect (int x1, int x2, int y1, int y2);
   void PaintRect(int x1, int x2, int y1, int y2, int Level);
   void Calc2w  (int x1, int x2, int y1, int y2, int Level);
   void Calc2s  (int x1, int x2, int y1, int y2, int Level);
   void Calc2(void);
   void MarkPoints(int xl, int yl, int xh, int yh); // fr FillQuad
   void AutoLogMap(void);
   void ZoomAnimation(int X, int Y);
};


class KMandel : public KTopLevelWidget
{
   Q_OBJECT
public:
    KMandel(const char *name=0 );
    ~KMandel();
    void SetClientSize(int Breite, int Hoehe);
    void SetStatus(char* Text, int ID); // Text in der Statuszeile ndern
    DrawWidget *Client;  // Clientbereich
    KStatusBar* StatusZeile;
protected:
    void resizeEvent( QResizeEvent * );
   void	keyPressEvent (QKeyEvent*);
private slots:
   void MenuNew();                    // Menpunkt "New"
   void MenuOpen();                   // Datei lesen
   void MenuWriteParams();            // Faktalparameter speichern
   void MenuFileSettings();           // Dateiparameter einstellen
   void ShowFracTypeDlg();            // Fraktaltpen-Dialog
   void ShowFracParamsDlg();          // Fraktalparameter (allgemein)
   void MenuSpecialParams();          // Typ-spezifische Einstellung
   void MenuParameterAnim();          // Parameteranimation
   void MenuPicSettings();            // generelle Bildeinstellung
   void MenuEditPal();                // Farbpalette bearbeiten
   void MenuLoadColorMap();           // Farbpalette laden
   void MenuSaveColorMap();           // Farbpalette speichern
   void MenuZoomSettings();           // Einstellungen Zoom
   void ShowAboutDlg();
   void MenuHelp();                   // Menpunkt Hilfe
private:
   KMenuBar* MainMenu;
   KToolBar* ToolBar;
   KConfig* Config;
   void InitMenue();
   void InitStatusZeile();
   void InitToolBar(void);
   void InitParameterRec(struct MandDataType* MD);// Parameterblock (typunabhngig) initialisieren
   void SetKeys(void);
   int  LoadParamFile(QString* DN);
};

#endif
