// p0 = Lambda.x 0.8
// p1 = Lambda.y  -0.6
#include <stdio.h>
#include <stdlib.h>
//#include <complex.h>
#include "l_julia.h"
#include "l_julia.moc"
#define TRS klocale->translate

double LJPar[2] = {0.85,-0.6};
void LJulia::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
//   fprintf(stdout,"LJulia.init\n");
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0;i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void LJulia::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -1.25+0.25;
   *x2 =  2.5-0.25;
   *y1 = -1.25;
   *y2 =  1.25;
}

double LJulia::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index>1) return 0.0;
   return LJPar[Index];
}

int LJulia::CalcPoint(int x, int y)
{
//   complex z,h,l;
   int iter;
   double zr,zi, hr,hi, lbr,lbi, h;

   zr = li+x*dx;
   zi = un+y*dy;             // lambda = Pixel
   lbr = p[0];  lbi = p[1];           // z(0) = 0;
   iter=0;

   do {
      hr = 1.0 - zr;   hi = -zi;  // 1-z(n);

      h  = lbr*zr - lbi*zi;         // Lambda*z(n)
      zi = lbr*zi + lbi*zr;
      zr = h;

      h  = zr*hr - zi*hi;         // Lambda*z(n)*(1-z(n))
      zi = zr*hi + zi*hr;
      zr = h;
      
      iter++;
   } while ((iter<MaxIter) && ConvTest(zr*zr,zi*zi));
   return PostCompute(iter,zr,zi);
}

char* LJulia::GetFracInfo(void) {
   return "Lambda=comp(p0,p1), x = Pixel;\n loop {\n    h=1-x\n    x=Lambda*x\n    x=x*h\n}\nuntil (x^2>Bail)";
}

int LJulia::SetupFrac(BaseFract* Frac) // type specific-setup
{
LJuliaDlg FDlg = LJuliaDlg(Frac, 0, "Lambda-Julia settings");

   if (FDlg.exec()) {
      return FDlg.SetSpecificParams();
   } else
      return FALSE;
}

// -----------------------------------------

LJuliaDlg::LJuliaDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[32];
   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Image Constant Values"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   ConstX = new QLineEdit(this, "ConstX");
   ConstX->setGeometry(100,50,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[0]);
   ConstX->setText(Strg);
   ConstY = new QLineEdit(this, "ConstY");
   ConstY->setGeometry(100,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[1]);
   ConstY->setText(Strg);

   QLabel* Label1 = new QLabel(TRS("ConstX"),this,"Label1");
   Label1->setGeometry(20,50,75,20);
   QLabel* Label2 = new QLabel(TRS("ConstY"),this,"Label2");
   Label2->setGeometry(20,80,75,20);

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefaultsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int LJuliaDlg::SetSpecificParams(void)
{
char* eptr;
double p0,p1;

   p0 = strtod(ConstX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p1 = strtod(ConstY->text(), &eptr);
   if (*eptr!=0) return FALSE;

   Fractal->p[0] = p0;
   Fractal->p[1] = p1;
   return TRUE;
}

void LJuliaDlg::SetDefaults()
{
char Strg[32];

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   ConstX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(1));
   ConstY->setText(Strg);
}