// Lambda-Mandelbrot
// p[0] = Start.x (0.5)
// p[1] = Start.y (0.0)
#include <stdio.h>
#include <stdlib.h>
//#include <complex.h>
#include "l_mandel.h"
#include "l_mandel.moc"
#define TRS klocale->translate

double LMPar[2]={0.5,0};
void LMandel::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
//   fprintf(stdout,"LMandel.init\n");
   SetDefaultCorners(&li, &re, &un, &ob);
   for (i=0;i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void LMandel::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -2.5;
   *x2 =  5;
   *y1 = -3.0;
   *y2 =  3.0;
}

double LMandel::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index>=2) return 0.0;
   return LMPar[Index];
}
int LMandel::CalcPoint(int x, int y)
{
//   complex z,h,l;
   int iter;
   double zr,zi, hr,hi, lbr,lbi, h;

   lbr = li+x*dx;
   lbi = un+y*dy;             // lambda = Pixel
   zr = p[0];  zi = p[1];           // z(0) = 0;
   iter=0;

   do {
      hr = 1.0 - zr;   hi = -zi;  // 1-z(n);

      h  = lbr*zr - lbi*zi;         // Lambda*z(n)
      zi = lbr*zi + lbi*zr;
      zr = h;

      h  = zr*hr - zi*hi;         // Lambda*z(n)*(1-z(n))
      zi = zr*hi + zi*hr;
      zr = h;
      
      iter++;
   } while ((iter<MaxIter) && ConvTest(zr*zr,zi*zi));
   return PostCompute(iter,zr,zi);

}

char* LMandel::GetFracInfo(void) {
   return "Lambda=Pixel, x = comp(p0,p1);\n loop {\n    h=1-x\n    x=Lambda*x\n    x=x*h\n}\nuntil (x^2>Bail)";
}

int LMandel::SetupFrac(BaseFract* Frac) // type specific-setup
{
LMandelDlg FDlg = LMandelDlg(Frac, 0, "Lambda-Mandelbrot settings");

   if (FDlg.exec()) {
      return FDlg.SetSpecificParams();
   } else
      return FALSE;
}


// -----------------------------------------

LMandelDlg::LMandelDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[32];
   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Image Constant Values"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   OffsetX = new QLineEdit(this, "OffsetX");
   OffsetX->setGeometry(100,50,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[0]);
   OffsetX->setText(Strg);
   OffsetY = new QLineEdit(this, "OffsetY");
   OffsetY->setGeometry(100,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[1]);
   OffsetY->setText(Strg);

   QLabel* Label1 = new QLabel(TRS("ConstX"),this,"Label1");
   Label1->setGeometry(20,50,75,20);
   QLabel* Label2 = new QLabel(TRS("ConstY"),this,"Label2");
   Label2->setGeometry(20,80,75,20);

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefaultsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int LMandelDlg::SetSpecificParams(void)
{
char* eptr;
double p0,p1;

   p0 = strtod(OffsetX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p1 = strtod(OffsetY->text(), &eptr);
   if (*eptr!=0) return FALSE;

   Fractal->p[0] = p0;
   Fractal->p[1] = p1;
   return TRUE;
}

void LMandelDlg::SetDefaults()
{
char Strg[32];

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   OffsetX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(1));
   OffsetY->setText(Strg);
}