// Fraktaltyp Stern
// p[0] = pixel.x
// p[1] = pixel.y
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mymath.h"
#include "lorenz.h"

double LorenzPar[4] = {5,15,1,0.02};

void Lorenz::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void Lorenz::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -15;
   *x2 =  15;
   *y1 =   0;
   *y2 =  30;
}

double Lorenz::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 4) return 0.0;
   return LorenzPar[Index];
}

void Lorenz::InitOrbit(void) {
   OrbX=0.1;
   OrbY=0.1;
   OrbZ=0.1;
}

void Lorenz::CalcOrbit(void)
// Berechnung fr orbitale Tpen
{
// Orbit Typ1 Version a
double h1,h2;

    h1   = OrbX + (-p[0] * OrbX * p[3])  +  (  p[0] * OrbZ * p[3]);
    h2   = OrbZ + ( p[1] * OrbX * p[3])  -  (         OrbZ * p[3])  -  ( OrbY * OrbX * p[3]);
    OrbY = OrbY + (-p[2] * OrbY * p[3])  +  (OrbX * OrbZ * p[3]);
    OrbX = h1;
    OrbZ = h2;
}


char* Lorenz::GetFracInfo(void)
{
   return 
"Orbit in three dimensions\n\n\
    X(n+1)  = X(n) + (-a * X(n) * d)  +  (  a  * Z(n) * d);\n\
    Z(n+1)  = Z(n) + ( b * X(n) * d)  -  (       Z(n) * d)  -  ( Y(n) * X(n) * d);\n\
    Y(n+1)  = Y(n) + (-c * Y(n) * d)  +  (X(n) * Z(n) * d);\n\
Parameters are a,b,c,d";
}

int Lorenz::SetupFrac(BaseFract* Frac) // type specific-setup
{
GenericDlg GDlg = GenericDlg(Frac, 0, "Lorenz settings");

   GDlg.SetHeader("Parameters a,b,c,d");
   GDlg.SetParameterNum(3);
   GDlg.SetParameterText(0,"a");
   GDlg.SetParameterText(1,"b");
   GDlg.SetParameterText(2,"c");
   GDlg.SetParameterText(3,"d");

   if (GDlg.exec()) {
      return GDlg.SetSpecificParams();
   } else
      return FALSE;
}
