// Fraktaltyp Mand4
// p[0] = pixel.x
// p[1] = pixel.y
#include <stdio.h>
#include <stdlib.h>
#include "mand4.h"
#define TRS klocale->translate

double Mand4Par[4] = {1.3, 1.5, -0.93, -1.8};

void Mand4::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void Mand4::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -2.5;     // Gre des Fraktals
   *x2 =  1.5;
   *y1 = -1.5;
   *y2 =  1.5;
}

double Mand4::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 4) return 0.0;
   return Mand4Par[Index];
}

int Mand4::CalcPoint(int x, int y)
{
int    iter=0;
double rx = li+dx*x;
double ry = un+dy*y;
complex<double> c,z;
complex<double> a,b,d,f,g,h,j;

   c=complex<double>(rx,ry);
   a=complex<double>(p[0],0);
   b=complex<double>(p[1],0);
   d=complex<double>(p[2],0);
   f=complex<double>(p[3],0);
   g=1/f; h=1/d; j=1/(f-b);
   z=pow(-a*b*g*h,j);
   do {
      z=(a*pow(z,b)) + (d*pow(z,f))+c;
      iter++;
   } while ((iter<MaxIter) && ConvTest(z.real()*z.real(),z.imag()*z.imag()) );
   return PostCompute(iter,z.real(),z.imag());
}

char* Mand4::GetFracInfo(void)
{
   return 
"Formula: c=Pixel; a=p0; b=p1; d=p2; f=p3\n\
         g=1/f; h=1/d; j=1/(f-b);\n\
         z=(-a*b*g*h)^j;\n\
         loop {\n\
            z=a*z^b + d*z^f +c\n\
         } until (z^2>Bail)";
}

int Mand4::SetupFrac(BaseFract* Frac) // type specific-setup
{
GenericDlg GDlg = GenericDlg(Frac, 0, "Mand4 settings");

   GDlg.SetHeader(TRS("Parameters p1,p2"));
   GDlg.SetParameterNum(3);
   GDlg.SetParameterText(0,"p1 real");
   GDlg.SetParameterText(1,"p1 imag");
   GDlg.SetParameterText(2,"p2 real");
   GDlg.SetParameterText(3,"p2 imag");

   if (GDlg.exec()) {
      return GDlg.SetSpecificParams();
   } else
      return FALSE;
}


