// Fraktaltyp Mandel hoch N
// p[0] = pixel.x
// p[1] = pixel.y
#include <stdio.h>
#include <stdlib.h>
#include <complex.h>
#include "mandel_n.h"
#include "mandel_n.moc"
#define TRS klocale->translate

double MandelHochNPar[4] = {0, 0.0, 3.0,0.0};

void MandelHochN::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void MandelHochN::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -2.0;     // Gre des Fraktals
   *x2 =  2.0;
   *y1 = -1.5;
   *y2 =  1.5;
}

double MandelHochN::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 4) return 0.0;
   return MandelHochNPar[Index];
}

int MandelHochN::CalcPoint(int x, int y)
{
//double c;
double rx = li+dx*x;
double ry = un+dy*y;
double a=rx+p[0];
double b=ry+p[1];
//double aa=a, bb=b;
int    iter=0;
//int Lauf;
complex<double> X,C,P;

   C=complex<double>(rx,ry);
   X=complex<double>(a,b);
   P=complex<double>(p[2],p[3]);
   
   do {
      X=pow(X,P);
      X+=C;
      iter++;
   } while ((iter<MaxIter) && ConvTest(X.real()*X.real(),X.imag()*X.imag()) );

/*   do {
      aa=a;   bb=b;
      for (Lauf=0; Lauf < l[0]; Lauf++) {
         c = a*aa - b*bb;
         b = a*bb+b*aa;
         a = c;
      }
      a += rx;
      b += ry;
      iter++;
   } while ((iter<MaxIter) && ConvTest(a*a,b*b) );
*/
   return PostCompute(iter,X.real(),X.imag());
}

char* MandelHochN::GetFracInfo(void)
{
   return "Formula: x=c=Pixel;\n     loop {x=x^N+c};\n     until (x^2>Bail)";
}

int MandelHochN::SetupFrac(BaseFract* Frac) // type specific-setup
{
MandelHochNDlg MDlg = MandelHochNDlg(Frac, 0, "Mandel hoch N settings");

   if (MDlg.exec()) {
      return MDlg.SetSpecificParams();
   } else
      return FALSE;
}


// -----------------------------------------

MandelHochNDlg::MandelHochNDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[32];
   Fractal = Frac;
   setFixedSize(300,240);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Image Constant Values"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   ConstX = new QLineEdit(this, "OffsetX");
   ConstX->setGeometry(100,50,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[0]);
   ConstX->setText(Strg);
   ConstY = new QLineEdit(this, "OffsetY");
   ConstY->setGeometry(100,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[1]);
   ConstY->setText(Strg);

   PotX = new QLineEdit(this, "PotX");
   PotX->setGeometry(150,110,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[2]);
   PotX->setText(Strg);

   PotY = new QLineEdit(this, "PotY");
   PotY->setGeometry(150,140,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[3]);
   PotY->setText(Strg);

   QLabel* Label1 = new QLabel(TRS("ConstX"),this,"Label1");
   Label1->setGeometry(20,50,75,20);
   QLabel* Label2 = new QLabel(TRS("ConstY"),this,"Label2");
   Label2->setGeometry(20,80,75,20);

   QLabel* PotX = new QLabel(TRS("Order Real Part"),this,"PotX");
   PotX->setGeometry(20,110,125,20);
   QLabel* PotY = new QLabel(TRS("Order Imaginary Part"),this,"PotY");
   PotY->setGeometry(20,140,125,20);
   
   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,190,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,190,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,190,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int MandelHochNDlg::SetSpecificParams(void)
{
char* eptr;
double p0,p1,p2,p3;
//long l0;

   p0 = strtod(ConstX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p1 = strtod(ConstY->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p2 = strtod(PotX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p3 = strtod(PotY->text(), &eptr);
   if (*eptr!=0) return FALSE;

   Fractal->p[0] = p0;
   Fractal->p[1] = p1;
   Fractal->p[2] = p2;
   Fractal->p[3] = p3;
   return TRUE;
}

void MandelHochNDlg::SetDefaults()
{
char Strg[32];

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   ConstX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(1));
   ConstY->setText(Strg);

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(2));
   PotX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(3));
   PotY->setText(Strg);
}