// Fraktaltyp MandelBrot
// p[0] = pixel.x
// p[1] = pixel.y
#include <stdio.h>
#include <stdlib.h>
#include "mandelbrot.h"
#include "mandelbrot.moc"
#define TRS klocale->translate

double MandelBrotPar[2] = {0, 0.0};

void MandelBrot::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void MandelBrot::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -2.5;     // Gre des Fraktals
   *x2 =  1.5;
   *y1 = -1.5;
   *y2 =  1.5;
}

double MandelBrot::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return MandelBrotPar[Index];
}

int MandelBrot::CalcPoint(int x, int y)
{
double c,d=0;
double rx = li+dx*x;
double ry = un+dy*y;
double a=rx+p[0];
double b=ry+p[1];
int    iter=0;
double txx,tyy,/*txy,*/newx,newy;

   if (l[0]) {
      do {
         c = a*a-b*b + rx;
         d=a*b;
         b = d+d + ry;
         //b = 2.0*a*a*b*b+ry;
         a = c;
         iter++;
      } while ((iter<MaxIter) && ConvTest(a*a,b*b) );

   } else {
   /* txx=txy=tyy=0.0;
   for (iter=0; iter<MaxIter; iter++) {
      newx=txx-tyy+rx;
      newy=txy+txy+ry;
      txx=newx*newx;
      tyy=newy*newy;
      txy=newx*newy;
      //if (txx+tyy>4.0) break; 
      if (!ConvTest(txx,tyy)) break;
   }*/

      newx=newy=txx=tyy=0.0;
      while ( (iter<MaxIter) && ConvTest(txx,tyy)) {
         c = txx-tyy+rx;
         newy=2.0*newx*newy+ry;
         newx=c;
         txx=newx*newx;
         tyy=newy*newy;
         iter++;
      }
      a=newx;b=newy;
   }
   return PostCompute(iter,a,b);
}

char* MandelBrot::GetFracInfo(void)
{
   return "Formula: x=c=Pixel;\n     loop {x=x^2+c};\n     until (x^2>Bail)";
}

int MandelBrot::SetupFrac(BaseFract* Frac) // type specific-setup
{
MandelBrotDlg MDlg = MandelBrotDlg(Frac, 0, "Mandelbrot settings");

   if (MDlg.exec()) {
      return MDlg.SetSpecificParams();
   } else
      return FALSE;
}


// -----------------------------------------

MandelBrotDlg::MandelBrotDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[32];
   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Image Constant Values"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   ConstX = new QLineEdit(this, "OffsetX");
   ConstX->setGeometry(100,50,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[0]);
   ConstX->setText(Strg);
   ConstY = new QLineEdit(this, "OffsetY");
   ConstY->setGeometry(100,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[1]);
   ConstY->setText(Strg);

   QLabel* Label1 = new QLabel(TRS("ConstX"),this,"Label1");
   Label1->setGeometry(20,50,75,20);
   QLabel* Label2 = new QLabel(TRS("ConstY"),this,"Label2");
   Label2->setGeometry(20,80,75,20);

   RegularCheck = new QCheckBox(TRS("Regular Fractal"), this, "RegularCheck");
   RegularCheck->setGeometry(20,110,150,20);
   RegularCheck->setChecked(Fractal->l[0]!=0);

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int MandelBrotDlg::SetSpecificParams(void)
{
char* eptr;
double p0,p1;
long l0;

   p0 = strtod(ConstX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p1 = strtod(ConstY->text(), &eptr);
   if (*eptr!=0) return FALSE;

   l0 = 0;
   if (RegularCheck->isChecked()) 
      l0 = 1; 

   Fractal->p[0] = p0;
   Fractal->p[1] = p1;
   Fractal->l[0] = l0;
   return TRUE;
}

void MandelBrotDlg::SetDefaults()
{
char Strg[32];

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   ConstX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(1));
   ConstY->setText(Strg);
   RegularCheck->setChecked(FALSE);
}