// Fraktaltyp Pocken
// l[0] Anzahl der Punkte (50)
// p[0] Intensity

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mymath.h"
#include "pocken.h"
#include "pocken.moc"
#define TRS klocale->translate
#define MAXPOINTS 500

double PockenPar[2] = {1, 0.0};

struct PointType {
          float X,Y;
	  float Value;
       };

struct PointType Points[MAXPOINTS];

void Pocken::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   l[0] = 50;
   InitPocken();
   SetSize(SizeX, SizeY);
}

void Pocken::InitPocken(void) {
int i;

fprintf(stdout,"initpocken\n");
   for (i=0;i<l[0];i++) {
      Points[i].X = frand();
      Points[i].Y = frand();
      Points[i].Value = frand()*2.0-1.0;
   }
}

void Pocken::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 =  0;     // Gre des Fraktals
   *x2 =  1;
   *y1 =  0;
   *y2 =  1;
}

double Pocken::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return PockenPar[Index];
}

int Pocken::CalcPoint(int x, int y)
{
double rx = li+dx*x;
double ry = un+dy*y;
int Lauf;
double r,entf,hilf;

   r = 0.0;
   for (Lauf=0; Lauf<l[0]; Lauf++) {
      /* Entferung bestimmen */
      entf = sqrt( sqr(rx-Points[Lauf].X) + sqr(ry-Points[Lauf].Y) );
      if (entf > 0.00005) {
         hilf = Points[Lauf].Value * p[0] / entf;
         r = r+hilf;
      }
   }
//   fprintf(stdout,"x=%d  y=%d   ret=%7.3f\n",x,y,r);
   return (int)r;
}

char* Pocken::GetFracInfo(void)
{
   return "?";
}

int Pocken::SetupFrac(BaseFract* Frac) // type specific-setup
{
PockenDlg MDlg = PockenDlg(Frac, 0, "Pocken settings");
int h;
   if (MDlg.exec()) {
      h = MDlg.SetSpecificParams();
      InitPocken();
      return h;
   } else
      return FALSE;
}

int Pocken::GetSpecialParamsBytes(void)
{
   return sizeof(Points);
}

void* Pocken::GetSpecialParamsBlock(void)
// Pointer auf Params, lesen
{
   return &Points;
}

void Pocken::PutSpecialParamsBlock(void)
{  // dummy
}


// -----------------------------------------

PockenDlg::PockenDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[20];

   Fractal = Frac;
   setFixedSize(350,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Pocks Parameter"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   PointsSpin = new KNumericSpinBox(this, "TransformSpin");
   PointsSpin->setGeometry(125,50,50,20);
   PointsSpin->setRange(5,500);
   PointsSpin->setValue(Fractal->l[0]);
   PointsSpin->setEditable(TRUE);

   QLabel *Label1 = new QLabel(TRS("#Pocks"), this, "Label1");
   Label1->setGeometry(20,50,90,20);
   
   QLabel *Label2 = new QLabel(TRS("Color Scale"), this, "Label2");
   Label2->setGeometry(20,80,90,20);
   
   ScaleEdit = new QLineEdit(this, "ScaleEdit");
   ScaleEdit->setGeometry(125,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[0]);
   ScaleEdit->setText(Strg);

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int PockenDlg::SetSpecificParams(void)
{
char* eptr;
double p0;

   p0 = strtod(ScaleEdit->text(), &eptr);
   if (*eptr!=0) return FALSE;

   Fractal->l[0] = PointsSpin->getValue();
   Fractal->p[0] = p0;
   return TRUE;
}

void PockenDlg::SetDefaults()
{
char Strg[20];

   PointsSpin->setValue(50);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   ScaleEdit->setText(Strg);
}

