// Fraktaltyp Triple
// p[0] = pixel.x
// p[1] = pixel.y
#include <stdio.h>
#include <stdlib.h>
#include "triple.h"
#include "triple.moc"
#define TRS klocale->translate

double TriplePar[2] = {0, 0.0};

void Triple::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   SetSize(SizeX, SizeY);
}

void Triple::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -2;     // Gre des Fraktals
   *x2 =  2;
   *y1 = -2;
   *y2 =  2;
}

double Triple::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return TriplePar[Index];
}

int Triple::CalcPoint(int x, int y)
{
double rx = li+dx*x;
double ry = un+dy*y;
double a=rx+p[0];
double b=ry+p[1];
int    iter=0;
double zr, zi ,h;

  zr = 0.0;   zi = 0.0;
  iter=0;
  do {
     h  = zr*zr - zi*zi;    // z
     zi = 2*zr*zi;
     zr = h;

     zr = -zr;      // z=-z; z=conj(z);

     zr += a;
     zi += b;       // z = z+Pixel

     iter++;
   } while ((iter<MaxIter) && ConvTest(zr*zr,zi*zi) );
   return PostCompute(iter,a,b);
}

char* Triple::GetFracInfo(void)
{
   return 
"Formula: z=0\n\
loop {\n\
   z = z^2\n\
   z = conj(z)\n\
   z = z+Pixel\n\
} until (|z| > bail)";
   
}

int Triple::SetupFrac(BaseFract* Frac) // type specific-setup
{
TripleDlg MDlg = TripleDlg(Frac, 0, "Triple settings");

   if (MDlg.exec()) {
      return MDlg.SetSpecificParams();
   } else
      return FALSE;
}


// -----------------------------------------

TripleDlg::TripleDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[32];
   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel(TRS("Offset Values of X(0)"), this, "Label0");
   Label0->setGeometry(30,20,150,20);

   ConstX = new QLineEdit(this, "OffsetX");
   ConstX->setGeometry(125,50,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[0]);
   ConstX->setText(Strg);
   ConstY = new QLineEdit(this, "OffsetY");
   ConstY->setGeometry(125,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[1]);
   ConstY->setText(Strg);

   QLabel* Label1 = new QLabel(TRS("Offset X"),this,"Label1");
   Label1->setGeometry(20,50,100,20);
   QLabel* Label2 = new QLabel(TRS("Offset Y"),this,"Label2");
   Label2->setGeometry(20,80,100,20);

   QPushButton* OkBtn = new QPushButton(TRS("OK"), this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton(TRS("Defaults"), this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton(TRS("Cancel"), this, "OkBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int TripleDlg::SetSpecificParams(void)
{
char* eptr;
double p0,p1;

   p0 = strtod(ConstX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p1 = strtod(ConstY->text(), &eptr);
   if (*eptr!=0) return FALSE;


   Fractal->p[0] = p0;
   Fractal->p[1] = p1;
   return TRUE;
}

void TripleDlg::SetDefaults()
{
char Strg[32];

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   ConstX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(1));
   ConstY->setText(Strg);
}